<?php
declare(strict_types=1);

namespace Inti\Wp\Provider\Metadata;

use Inti\Provider\AbstractProvider;

use function in_array;

/**
 * Class AbstractProvider
 *
 * @package Inti\Wp\Provider\Metadata
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-provider
 */
abstract class AbstractMetadataProvider extends AbstractProvider implements MetadataProviderInterface
{
    protected array $meta_keys = [];
    protected array $metadata  = [];

    protected function fetchResults(): array
    {
        return [];
    }

    /** @noinspection PhpUnusedParameterInspection */
    protected function fetchMetadata(int $id): array
    {
        return [];
    }

    protected function mapResultsToItems(array $results): array
    {
        $items = parent::mapResultsToItems($results);

        foreach ($items as &$item) {
            $itemId = $this->getId($item);

            foreach ($item as $key => $value) {
                if (in_array($key, $this->meta_keys)) {
                    $this->metadata[$itemId][$key] = $value;
                    unset($item[$key]);
                }
            }
        }

        return $items;
    }

    public function hasMetadataKey(string $key): bool
    {
        return in_array($key, $this->meta_keys);
    }

    public function getMetadata(int $id): array
    {
        if (!isset($this->metadata[$id])) {
            $this->metadata[$id] = $this->fetchMetadata($id);
        }

        return $this->metadata[$id] ?? [];
    }
}
