<?php
declare(strict_types=1);

namespace Inti\Wp\Provider\Type;

use Inti\Provider\Type\AbstractType;

/**
 * Class AbstractObjectType
 *
 * @package Inti\Wp\Provider\Type
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-provider
 *
 * @method static WpTypeInterface instance()
 */
abstract class AbstractWpType extends AbstractType implements WpTypeInterface
{
    protected array  $capabilities       = [];
    protected string $description        = '';
    protected bool   $hierarchical       = false;
    protected array  $labels             = [];
    /** @var callable|null */
    protected        $meta_box_cb        = null;
    protected bool   $public             = false;
    protected ?bool  $publicly_queryable = null;
    /** @var bool|string */
    protected        $query_var          = true;
    /** @var bool|array */
    protected        $rewrite            = true;
    protected ?bool  $show_in_menu       = null;
    protected ?bool  $show_in_nav_menus  = null;
    protected ?bool  $show_ui            = null;

    public function getCapabilities(): array
    {
        return $this->capabilities;
    }

    public function getDescription(): string
    {
        return $this->description;
    }

    public function getLabels(): array
    {
        return $this->labels;
    }

    public function getMetaBoxCallback(): ?callable
    {
        return $this->meta_box_cb;
    }

    public function getQueryVar()
    {
        return $this->query_var;
    }

    public function getRewriteConfig()
    {
        return $this->rewrite;
    }

    public function isBuiltin(): bool
     {
         return false;
     }

    public function isHierarchical(): bool
    {
        return $this->hierarchical;
    }

    public function isPublic(): bool
    {
        return $this->public;
    }

    public function isPubliclyQueryable(): ?bool
    {
        return $this->publicly_queryable;
    }

    /**
     * @return bool|null
     */
    public function showInMenu(): ?bool
    {
        return $this->show_in_menu;
    }

    public function showInNavMenus(): ?bool
    {
        return $this->show_in_nav_menus;
    }

    public function showUi(): ?bool
    {
        return $this->show_ui;
    }
}
