<?php
declare(strict_types=1);

namespace IntiTest\Wp\Provider\Mock;

use Inti\Provider\Query\QueryTrait;
use Inti\Provider\Type\TypeInterface;
use Inti\Wp\Provider\Term\AbstractTermProvider;

use function preg_match;
use function preg_quote;

/**
 * Class TermProvider
 *
 * @package IntiTest\Wp\Provider\Mock
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-provider
 */
class TermProvider extends AbstractTermProvider
{
    use QueryTrait;
    use MetadataProviderTrait;

    protected array $field_map = [
        'id' => 'term_id',
    ];

    private array $items = [
        ['term_id' => 1, 'name' => 'First term', 'slug' => 'first-term'],
        ['term_id' => 2, 'name' => 'Second term', 'slug' => 'second-term'],
        ['term_id' => 3, 'name' => 'Third term', 'slug' => 'third-term'],
        ['term_id' => 4, 'name' => 'Fourth term', 'slug' => 'fourth-term'],
        ['term_id' => 5, 'name' => 'Fifth term', 'slug' => 'fifth-term'],
        ['term_id' => 6, 'name' => 'Sixth term', 'slug' => 'sixth-term'],
        ['term_id' => 7, 'name' => 'Seventh term', 'slug' => 'seventh-term'],
        ['term_id' => 8, 'name' => 'Eighth term', 'slug' => 'eighth-term'],
    ];

    public function __construct()
    {
        $this->initMetadata('term', 8, 2);
    }

    public function getProvidedType(): TypeInterface
    {
        return CustomTaxonomy::instance();
    }

    protected function fetchResults(): array
    {
        $results = array_filter($this->items, function (array $item) {
            if (isset($this->_query['ids']) && !empty($this->_query['ids'])) {
                $itemId = $this->getId($item);
                return in_array($itemId, $this->_query['ids']);
            }

            if (isset($this->_query['search']) && !empty($this->_query['search'])) {
                $query = preg_quote($this->_query['search']);
                return preg_match("/{$query}/", $item['name']);
            }

            if (isset($this->_query['slugs']) && !empty($this->_query['slugs'])) {
                return in_array($item['slug'], $this->_query['slugs']);
            }

            return true;
        });

        if (isset($this->_query['paginate'])) {
            [$page, $itemsPerPage] = $this->_query['paginate'];
            $results = array_chunk($results, $itemsPerPage);
            $results = $results[$page - 1];
        }

        return $results;
    }

    public function getCount(): int
    {
        return count($this->items);
    }
}
