<?php
/** @noinspection PhpUnusedPrivateFieldInspection */
declare(strict_types=1);

namespace IntiTest\Wp\Provider\Plugin;

use Inti\Provider\Service\ProviderManager;
use IntiTest\Wp\Provider\AbstractTestCase;
use IntiTest\Wp\Provider\Mock;
use WP_Taxonomy;
use WP_Term;

use function get_categories;
use function get_term_by;
use function get_terms;
use function wp_count_terms;

/**
 * Class ProvideTermsTest
 *
 * @package IntiTest\Wp\Provider\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-provider
 *
 * @runTestsInSeparateProcesses
 */
final class ProvideTermsTest extends AbstractTestCase
{
    private array $config = [
        ProviderManager::class => [
            'registered' => [
                Mock\TermProvider::class,
                Mock\OverrideTermProvider::class,
            ],
        ],
    ];

    public function testWillRegisterCustomTaxonomy()
    {
        $taxonomy = get_taxonomy('custom_taxonomy');
        $this->assertInstanceOf(WP_Taxonomy::class, $taxonomy);

        $this->assertEquals('Custom Terms', $taxonomy->label);
        $this->assertEquals('some description', $taxonomy->description);
        $this->assertFalse($taxonomy->public);
        $this->assertEquals(['custom_post'], $taxonomy->object_type);
    }

    public function testCanProvideTerms()
    {
        $terms = get_terms([
            'taxonomy' => 'custom_taxonomy'
        ]);

        $this->assertCount(8, $terms);
    }

    public function testCanGetTermsByIds()
    {
        $terms = get_terms([
            'include' => [100000002],
            'taxonomy' => 'custom_taxonomy',
        ]);

        $this->assertCount(1, $terms);
        $this->assertEquals('Second term', $terms[0]->name);
    }

    public function testCanCountTerms()
    {
        $count = wp_count_terms([
            'taxonomy' => 'custom_taxonomy',
        ]);

        $this->assertEquals(8, $count);
    }

    public function testCanProvideTermsToExistingTaxonomies()
    {
        $terms = get_terms([
            'taxonomy' => 'category',
        ]);

        $this->assertCount(8, $terms);
    }

    public function testCanGetCategories()
    {
        $categories = get_categories([
            'parent' => 0,
            'hide_empty' => 0,
            'hierarchical' => 1,
            'taxonomy' => 'category',
            'pad_counts' => 1,
        ]);

        $this->assertCount(8, $categories);
    }

    public function testCanGetTermBySlug()
    {
        $term = get_term_by('slug', 'second-term', 'custom_taxonomy');
        $this->assertInstanceOf(WP_Term::class, $term);
        $this->assertEquals('second-term', $term->slug);
    }
}
